/* 
 *  ==========================================================================
 *  PassengerDataModel.java: Simple data model for storing list of passengers
 *                           on a vehicle. 
 * 
 *  Written by: Mark Austin                                         April 2022 
 *  ==========================================================================
 */ 

package homework03;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.CSVWriter;

import java.lang.Math;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Date;

import java.time.LocalDate;

public class PassengerDataModel {
   String dataFileName;
   String  vehicleName;
   boolean DEBUG = true;
   int count = 0;

   // Arraylist passenger data items ...

   private ArrayList<PassengerDataItem> passengerData;

   // Hashmap for key = passenger name, value = passenger data items ...

   private HashMap<String,PassengerDataItem> namePassengerDataItemMap;

   // Default constructor method ...

   public PassengerDataModel() {
      passengerData = new ArrayList<PassengerDataItem>();
      namePassengerDataItemMap = new HashMap<String,PassengerDataItem>();
   };

   // Set vehicle name ...

   public void setName( String name ) {
      this.vehicleName = name;
   }

   // Load data file into model ...

   public void loadDataFile(String dataFileName ) throws IOException {
      this.dataFileName = dataFileName;

      // Open CSVReader and read headers of first row ...

      System.out.printf("--- Read column headers off first row ...\n" );

      CSVReader reader = new CSVReader( new FileReader( dataFileName ));
      String headerLine [] = reader.readNext();

      for (int i = 0; i < headerLine.length; i = i + 1) {
         System.out.printf("Column %d : %s \n", i + 1, headerLine[i] );
      }

      // Loop over passenger list ...

      String [] nextLine;
      while ((nextLine = reader.readNext()) != null) {
         count = count + 1;

         System.out.printf("--- Passenger: %d ...\n", count );

         // Assemble energy data item object ...

         PassengerDataItem item = new PassengerDataItem();
         for (int i = 0; i < headerLine.length; i = i + 1) {
             item.add ( headerLine [i], nextLine[i] );
         }

         // Add passenger data item to array list ...

         passengerData.add (item );

         // Add passenger data item to hashmap ...

         String passengerName = nextLine[2];
         namePassengerDataItemMap.put( passengerName, item );

         System.out.println( item.toString() );
      }
   }

   // =====================================================
   // Export data model to arff file format ...
   // =====================================================

   public void exportARFF( String arffFileName ) throws IOException {

      System.out.printf("--- Enter exportARFF () ...\n");
      System.out.printf("--- ARFF datafile: %s ...\n", arffFileName );

      // Check that file name has .arff extension ....

      if ( arffFileName.contains(".")) {
         String[] parts = arffFileName.split("\\.");
         String extension = (String) parts[ parts.length - 1 ];

         if (DEBUG == true) {
             System.out.printf("--- Filename extension: %s ... \n", extension );
         }

         if (extension.contains("arff") != true) {
            System.out.printf("--- Error: WeatherDataModel.exportARFF() ... \n" );
            System.out.printf("--- Error: Filename extension needs to be \".arff\"! ...\n" );
         }
      } else {
         throw new IllegalArgumentException("String " + arffFileName + " does not contain dot (.)");
      }

      // Create arff file ...

      Writer fw = null;
      try {
         fw = new FileWriter( arffFileName );

         // Write file header info

         String header01 = "% ================================================= ... \n";
         String header02 = "% Passenger data file                   April, 2022 ... \n";
         String header03 = "% ================================================= ... \n";

         fw.write( header01 );
         fw.write( header02 );
         fw.write( header03 );

         // =======================================================
         // HOMEWORK 3: Fill in details here .... for hints, see:
         // 
         // /src/arraylist06/DemoChicagoEnergyBenchmarking.java ... 
         // =======================================================

         // ... fill in details ...

         // ... fill in details ...

         // ... fill in details ...

      } catch (IOException e) {}
      finally {
         try { fw.close(); }
         catch (Exception e) {}
      }

      System.out.printf("--- Leave exportARFF() ...\n");
   }

   // String description of passenger data model ...

   public String toString() {
      StringBuffer buf = new StringBuffer();

      buf.append( String.format("PassengerDataModel ... \n"));
      buf.append( String.format("======================================= ... \n"));
      buf.append( String.format("--- Data file: %s ... \n", dataFileName ));
      buf.append( String.format("--- Passenger Vehicle: %s ... \n", vehicleName ));
      buf.append( String.format("--- No passengers: %d ... \n", count ));
      buf.append( String.format("======================================= ... \n"));

      return buf.toString();
   }
}
