/**
 * ===================================================================================
 * SymbolTableFactory.java: A factory for creating objects that implement the symbol table.
 *
 * Copyright (c) 2009 by Ronald Mak
 * For instructional purposes only.  No warranties.
 * ===================================================================================
 */

package symbol02;

public class SymbolTableFactory {

    /**
     * Create and return a symbol table stack implementation.
     * @return the symbol table implementation.
     */

    public static SymbolTableStack createSymbolTableStack() {
        return new SymbolTableStackImpl();
    }

    /**
     * Create and return a symbol table implementation.
     * @param nestingLevel the nesting level.
     * @return the symbol table implementation.
     */

    public static SymbolTable createSymbolTable(int nestingLevel) {
        return new SymbolTableImpl(nestingLevel);
    }

    /**
     * Create and return a symbol table entry implementation.
     * @param name the identifier name.
     * @param symTab the symbol table that contains this entry.
     * @return the symbol table entry implementation.
     */

    public static SymbolTableEntry createSymbolTableEntry(String name, SymbolTable symTab) {
        return new SymbolTableEntryImpl(name, symTab);
    }

    // =================================================================
    // Exercise methods in Symbol Table Factory ....
    // =================================================================

    public static void main ( String args[] ) {

       System.out.println("");
       System.out.println("*** Exercise SymbolTableFactory ... ");
       System.out.println("*** =============================== ");
       System.out.println("");

       SymbolTableFactory stf = new SymbolTableFactory();
       SymbolTableStack   sts = stf.createSymbolTableStack();

       SymbolTable st00  = stf.createSymbolTable(0);
       st00.enter("pi");
       st00.enter("b string");
       st00.enter("c string");
       st00.enter("d string");

       String targetName = "pi";
       SymbolTableEntry target01 = st00.lookup(targetName);
       target01.setAttribute( SymbolTableKeyImpl.CONSTANT, new Double(3.141) );
       targetName = "b string";
       SymbolTableEntry target02 = st00.lookup(targetName);
       target02.setAttribute( SymbolTableKeyImpl.VAR, new Integer(2) );
       targetName = "c string";
       SymbolTableEntry target03 = st00.lookup(targetName);
       target03.setAttribute( SymbolTableKeyImpl.DATA, new Integer(2) );
       targetName = "d string";
       SymbolTableEntry target04 = st00.lookup(targetName);
       target04.setAttribute( SymbolTableKeyImpl.DATA, new Integer(8) );

       System.out.println ( "*** Target01: " + target01.toString() );
       System.out.println ( "***   Name01: " + target01.getName()  );
       System.out.println ( "*** Target02: " + target02.toString() );
       System.out.println ( "***   Name02: " + target02.getName()  );
       System.out.println ( "*** Target03: " + target03.toString() );
       System.out.println ( "***   Name03: " + target03.getName()  );
       System.out.println ( "*** Target04: " + target04.toString() );
       System.out.println ( "***   Name04: " + target04.getName()  );

       SymbolTable st01  = stf.createSymbolTable(1);
       st01.enter("0 string");
       st01.enter("1 string");
       st01.enter("5 string");
       st01.enter("2 string");
       st01.enter("6 string");
       SymbolTable st02  = stf.createSymbolTable(2);
       st02.enter("xyz string");

       sts.push ( st00 );
       sts.push ( st01 );
       sts.push ( st02 );

       targetName = "b string";
       SymbolTableEntry targetId = sts.lookup(targetName);
       if (targetId == null) {
           System.out.println("*** targetID not found! ");
           targetId = sts.enterLocal(targetName);
       } 

       // Print contents of symbol table ...

       System.out.println ( st00.sortedEntries().toString() );
       System.out.println ( st01.sortedEntries().toString() );
       System.out.println ( st02.sortedEntries().toString() );

    }
}
