/*
 * ======================================================================
 * Dimensions.java:
 *
 * Written by: Mark Austin
 * ======================================================================
 */

package symbol02.datatypes;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

public class Dimensions {
   private String        sName;  // units name 
   private double  scaleFactor;  // scale/conversion factor 
   private double  lengthExpnt;  // length                 
   private double    massExpnt;  // mass                  
   private double    timeExpnt;  // time  
   private double    tempExpnt;  // temperature
   private double  radianExpnt;  // planar angle 
   private DimensionsType type;  // US or SI units 

   // Create singleton for library of builtin units ...

   public static HashMap<String,Dimensions> library = new HashMap<String,Dimensions>();

   // Constructor methods ...

   public Dimensions() {
      this.type  = DimensionsType.SI;
   }

   public Dimensions( String sName ) {
      this.sName = sName;
      this.type  = DimensionsType.SI;
   }

   public Dimensions( String sName, double scaleFactor, double  lengthExpnt,
                      double massExpnt, double timeExpnt, double  tempExpnt,
                      double radianExpnt, DimensionsType type ) {
      this.sName       = sName;
      this.scaleFactor = scaleFactor;  // scale factor ..
      this.lengthExpnt = lengthExpnt;  // length                 
      this.massExpnt   = massExpnt;    // mass                 
      this.timeExpnt   = timeExpnt;    // time 
      this.tempExpnt   = tempExpnt;    // temperature
      this.radianExpnt = radianExpnt;  // radian
      this.type        = type;         // dimensions type ...
   }

   // Set/get dimensions name ...

   public void setName ( String sName ) {
      this.sName = sName;
   }

   public String getName () {
      return sName;
   }

   // Set/get dimensions type ...

   public void setUnitsType ( DimensionsType type ) {
      this.type = type;
   }

   public DimensionsType getUnitsType () {
      return type;
   }

   // ======================================================
   // Computational units and dimensions ...
   // ======================================================

   public boolean sameDimensions( Dimensions temp ) {
      boolean result = false;

      if((this.scaleFactor  == temp.scaleFactor) &&
         (this.lengthExpnt  == temp.lengthExpnt) &&
         (this.massExpnt    == temp.massExpnt)   &&
         (this.timeExpnt    == temp.timeExpnt)   &&
         (this.tempExpnt    == temp.tempExpnt) )  {
          result = true;
      }

      return result;
   }

   public boolean sameUnits( Dimensions temp ) {
      boolean result = false;

      if((this.lengthExpnt  == temp.lengthExpnt) &&
         (this.massExpnt    == temp.massExpnt)   &&
         (this.timeExpnt    == temp.timeExpnt)   &&
         (this.tempExpnt    == temp.tempExpnt) )  {
          result = true;
      }
    
      return result;
   }

   // ======================================================
   // Computational methods ...
   // ======================================================

   public Dimensions mult( Dimensions a ) {
      Dimensions temp = new Dimensions();
      return temp;
   }

   public Dimensions multRep( Dimensions a, Dimensions b, Dimensions c ) {
      Dimensions temp = new Dimensions();
      return temp;
   }

   public Dimensions divide( Dimensions a, Dimensions b, int x ) {
      Dimensions temp = new Dimensions();
      return temp;
   }

   public Dimensions divideRep( Dimensions a, Dimensions b, Dimensions c, int y ) {
      Dimensions temp = new Dimensions();
      return temp;
   }

   public Dimensions power( Dimensions a, double d , int z ) {
      Dimensions temp = new Dimensions();
      return temp;
   }

   public Dimensions powerRep( Dimensions a, Dimensions b, double d , int z ) {
      Dimensions temp = new Dimensions();
      return temp;
   }

   public Dimensions copy( Dimensions a, Dimensions b ) {
      Dimensions temp = new Dimensions();
      return temp;
   }

   // Zero out units and name ....

   public void zeroUnits() {
      this.sName       = "";
      this.scaleFactor = 1.0;
      this.lengthExpnt = 0.0;
      this.massExpnt   = 0.0;
      this.timeExpnt   = 0.0;
      this.tempExpnt   = 0.0;
      this.radianExpnt = 0.0;
      if(this.type != DimensionsType.SI && this.type != DimensionsType.US )
         this.type = DimensionsType.SI_US;
   }

   // Set default units according to a name ...

   public void defaultUnits( String name ) {
   }

   // Convert units type ....

   public Dimensions unitsTypeConvert( DimensionsType type ) {
      Dimensions result = null;

      if( this.type != type || type != DimensionsType.SI_US ) {
         result = new Dimensions();
         result.setUnitsType( type );
         result.setName( null ); 
         // UnitsSimplify(d);
      } 

      return result;
   }

   // Simplify units ...

   public Dimensions simplify() {
      Dimensions temp = new Dimensions();
      return temp;
   }

   public Dimensions radUnitsSimplify() {
      Dimensions temp = new Dimensions();
      return temp;
   }

   // String representation of dimensions ...

   public String toString() {
      StringBuffer buf = new StringBuffer();
      buf.append( String.format("Dimension: %s\n",  sName ) );
      buf.append( String.format("Scale factor   = %10.2g\n", scaleFactor ) );
      buf.append( String.format("Expnts: length = %10.2f\n", lengthExpnt ) );
      buf.append( String.format("      :   mass = %10.2f\n", massExpnt ) );
      buf.append( String.format("      :   time = %10.2f\n", timeExpnt ) );
      buf.append( String.format("      :   temp = %10.2f\n", tempExpnt ) );
      buf.append( String.format("      : radian = %10.2f\n", radianExpnt ) );
      switch ( type ) {
          case SI:
               buf.append( String.format("Enum Type      = SI\n" ));
               break;
          case US:
               buf.append( String.format("Enum Type      = US\n" ));
               break;
          case SI_US:
               buf.append( String.format("Enum Type      = SI_US\n" ));
          default:
               break;
      }

      return buf.toString();
   }

   // =========================================================
   // Exercise methods in Dimensions ....
   // =========================================================

   public static void main( String args [] ) {

      // Part 01: Create and print a simple dimension ....

      System.out.println("");
      System.out.println("*** Exercise methods in aladdin/Dimensions ...");
      System.out.println("*** ========================================== ");

      Dimensions d1 = new Dimensions ( "d1" );
      System.out.println( d1.toString() );

      // Part 02: Build library of well-known unit names and their properties ...

      DimensionsLibrary lib = new DimensionsLibrary();
      HashMap library = lib.getLibrary();

      System.out.println("");
      System.out.println("*** Test equality of dimensions ...");
      System.out.println("*** ----------------------------------");
      System.out.println("");

      Dimensions d01 = (Dimensions) library.get("m");
      Dimensions d02 = (Dimensions) library.get("mm");

      if ( d01.sameDimensions (d02) == true )
         System.out.println("*** Dimensions d01 and d02 are equal!");
      else
         System.out.println("*** Dimensions d01 and d02 are different!");

      if ( d01.sameUnits (d02) == true )
         System.out.println("*** d01 and d02 have the same units!");
      else
         System.out.println("*** d02 and d02 have different units!");

   }
}
